﻿var first = 24;
var second = 2;

function demoDivideUsingCallback() {
    write('');
    write('Start calculation using callback...');

    run(first, second);
    run(first, 0);

    write('...calculation end (maybe)');

    function run(firstValue, secondValue) {
        write('Dividing ' + firstValue + ' by ' + secondValue);
        divideUsingCallback(firstValue, secondValue,
            function (result) {
                write('Well after... ' + firstValue + ' / ' + secondValue + ' = ' + result);
            },
            function (reason) {
                write('Well after... ' + firstValue + ' / ' + secondValue + ' = ' + reason);
            }
        );

    }
}

function demoDivideUsingPromise() {
    write('');
    write('Start calculation using promise...');

    run(first, second);
    run(first, 0);

    function run(firstValue, secondValue) {
        write('Dividing ' + firstValue + ' by ' + secondValue);
        divideUsingPromise(firstValue, secondValue)
            .then(
                function (result) {
                    write('Well after... ' + firstValue + ' / ' + secondValue + ' = ' + result);
                },
                function (reason) {
                    write('Well after... ' + firstValue + ' / ' + secondValue + ' = ' + reason);
                }
            );
    }

    write('...calculation end (maybe)');
}

function demoDivideEncapsulatingCallback() {
    write('');
    write('Start calculation encapsulating callback...');

    run(first, second);
    run(first, 0);

    function run(firstValue, secondValue) {
        write('Dividing ' + firstValue + ' by ' + secondValue);
        divideEncapsulatingCallback(firstValue, secondValue)
            .then(
                function (result) {
                    write('Well after... ' + firstValue + ' / ' + secondValue + ' = ' + result);
                },
                function (reason) {
                    write('Well after... ' + firstValue + ' / ' + secondValue + ' = ' + reason);
                }
            );
    }

    write('...calculation end (maybe)');
}

function demoChainingUsingCallback() {
    write('');
    write('Start processing using callback...');

    run('Giorgio');
    run();

    write('...processing end (maybe)');

    function run(name) {
        var viewModel = null;
        write('Getting data for ' + name);
        getCustomerWithCallback(name,
            function (customer) {
                viewModel = customer;
                getRecentOrdersWithCallback(customer.id,
                    function (orders) {
                        viewModel.recentOrders = orders;
                        getSuggestionsFromOrdersWithCallback(orders,
                            function (products) {
                                viewModel.suggestedProducts = products;
                                write(viewModel.name + ' has ' + viewModel.recentOrders.length + ' recent orders and ' + viewModel.suggestedProducts.length + ' suggested products');
                            },
                            function (reason) {
                                write('Getting data for ' + name + ' ends with ' + reason);
                            }
                        );
                    },
                    function (reason) {
                        write('Getting data for ' + name + ' ends with ' + reason);
                    }
                );
            },
            function (reason) {
                write('Getting data for ' + name + ' ends with ' + reason);
            }
        );
    }
}

function demoChainingUsingPromise() {
    write('');
    write('Start processing using promise...');

    run('Giorgio');
    run();

    write('...processing end (maybe)');

    function run(name) {
        var viewModel = null;
        write('Getting data for ' + name);
        getCustomerWithPromise(name)
            .then(function (customer) {
                viewModel = customer;
                return getRecentOrdersWithPromise(customer.id);
            })
            .then(function (orders) {
                viewModel.recentOrders = orders;
                return getSuggestionsFromOrdersWithPromise(orders);
            })
           .then(function (products) {
               viewModel.suggestedProducts = products;
               write(viewModel.name + ' has ' + viewModel.recentOrders.length + ' recent orders and ' + viewModel.suggestedProducts.length + ' suggested products');
            })
            .fail(
                function (reason) {
                    write('Getting data for ' + name + ' ends with ' + reason);
                }
            );
    }
}

function demoAllPromises() {
    write('');
    write('Start processing using promise in parallel...');

    run('Giorgio', 1);
    run();
    run('Roberto', null);

    write('...processing end (maybe)');

    function run(name, customerId) {
        var viewModel = null;
        write('Getting data for ' + name);
        getCustomerWithPromise(name)
            .then(function (customer) {
                viewModel = customer;
                return Q.all([
                    getRecentOrdersWithPromise(customerId),
                    getSuggestionsForCustomerWithPromise(customerId),
                ])
            })
            //.then(function (result) {
            //    viewModel.recentOrders = result[0];
            //    viewModel.suggestedProducts = result[1];
            //    write(viewModel.name + ' has ' + viewModel.recentOrders.length + ' recent orders and ' + viewModel.suggestedProducts.length + ' suggested products');
            //})
            .spread(function (orders, products) {
                viewModel.recentOrders = orders;
                viewModel.suggestedProducts = products;
                write(viewModel.name + ' has ' + viewModel.recentOrders.length + ' recent orders and ' + viewModel.suggestedProducts.length + ' suggested products');
            })
            .fail(
                function (reason) {
                    write('Getting data for ' + name + ' ends with ' + reason);
                }
            )
        .done();
    }
}

function demoWithoutDone() {
    write('');
    write('Start calculation without done()...');

    run(first, 0);

    function run(firstValue, secondValue) {
        write('Dividing ' + firstValue + ' by ' + secondValue);
        divideUsingPromise(firstValue, secondValue)
            .then(
                function (result) {
                    write('Well after... ' + firstValue + ' / ' + secondValue + ' = ' + result);
                }
            );
    }

    write('...calculation end (maybe)');
}

function demoWithDone() {
    write('');
    write('Start calculation with done()...');

    run(first, 0);

    function run(firstValue, secondValue) {
        write('Dividing ' + firstValue + ' by ' + secondValue);
        divideUsingPromise(firstValue, secondValue)
            .then(
                function (result) {
                    write('Well after... ' + firstValue + ' / ' + secondValue + ' = ' + result);
                }
            )
            .done();
    }

    write('...calculation end (maybe)');
}

function demoES6() {
    write('');
    if (window.Promise) {
        write('Promise available');

        var promise = new Promise(function (resolve, reject) {
            var request = new XMLHttpRequest();

            request.open('GET', 'http://localhost:59844/DemoPage.html');

            request.onload = function () {
                if (request.status == 200) {
                    resolve(request.response);
                } else {
                    reject(Error(request.statusText));
                }
            };

            request.onerror = function () {
                reject(new Error('Error fetching data.'));
            };

            request.send();
        });

        promise
            .then(function (data) {
                write('Got data! Promise fulfilled.');
                return data;
            }, function (error) {
                write(error.message);
            });
    } else {
        write('Promise not available');
    }
}